import subprocess
from typing import Optional, Union

from loguru import logger

from netspresso.benchmarker import BenchmarkerV2
from netspresso.clients.auth import TokenHandler, auth_client
from netspresso.clients.auth.response_body import UserResponse
from netspresso.clients.tao import TAOTokenHandler
from netspresso.compressor import CompressorV2
from netspresso.converter import ConverterV2
from netspresso.enums import Task
from netspresso.qai_hub.benchmarker import QAIHubBenchmarker
from netspresso.qai_hub.converter import QAIHubConverter
from netspresso.tao import TAOTrainer
from netspresso.trainer import Trainer


class NetsPresso:
    def __init__(self, email: str, password: str, verify_ssl: bool = True) -> None:
        """Initialize NetsPresso instance and perform user authentication.

        Args:
            email (str): User's email for authentication.
            password (str): User's password for authentication.
            verify_ssl (bool): Flag to indicate whether SSL certificates should be verified. Defaults to True.
        """
        self.token_handler = TokenHandler(
            email=email, password=password, verify_ssl=verify_ssl
        )
        self.user_info = self.get_user()

    def get_user(self) -> UserResponse:
        """Get user information using the access token.

        Returns:
            UserInfo: User information.
        """
        user_info = auth_client.get_user_info(
            self.token_handler.tokens.access_token, self.token_handler.verify_ssl
        )
        return user_info

    def trainer(
        self, task: Optional[Union[str, Task]] = None, yaml_path: Optional[str] = None
    ) -> Trainer:
        """Initialize and return a Trainer instance.

        Args:
            task (Union[str, Task], optional): Type of task (classification, detection, segmentation).
            yaml_path (str, optional): Path to the YAML configuration file.

        Returns:
            Trainer: Initialized Trainer instance.
        """
        return Trainer(token_handler=self.token_handler, task=task, yaml_path=yaml_path)

    def compressor_v2(self) -> CompressorV2:
        """Initialize and return a Compressor instance.

        Returns:
            Compressor: Initialized Compressor instance.
        """
        return CompressorV2(token_handler=self.token_handler)

    def converter_v2(self) -> ConverterV2:
        """Initialize and return a Converter instance.

        Returns:
            Converter: Initialized Converter instance.
        """
        return ConverterV2(token_handler=self.token_handler, user_info=self.user_info)

    def benchmarker_v2(self) -> BenchmarkerV2:
        """Initialize and return a Benchmarker instance.

        Returns:
            Benchmarker: Initialized Benchmarker instance.
        """
        return BenchmarkerV2(token_handler=self.token_handler, user_info=self.user_info)


class TAO:
    def __init__(self, ngc_api_key: str) -> None:
        """Initialize TAO instance and perform user authentication.

        Args:
            ngc_api_key (str): API key for TAO authentication.
        """
        self.ngc_api_key = ngc_api_key
        self.token_handler = TAOTokenHandler(ngc_api_key=ngc_api_key)

    def trainer(self) -> TAOTrainer:
        """Initialize and return a Trainer instance.

        Returns:
            TAO: Initialized Trainer instance.
        """
        return TAOTrainer(token_handler=self.token_handler)


class QAIHub:
    def __init__(self, api_token: str) -> None:
        # Define the command and arguments
        command = 'qai-hub'
        args = ['configure', '--api_token', f'{api_token}']

        # Execute the command
        result = subprocess.run([command] + args, capture_output=True, text=True)
        logger.info(result)

    def converter(self) -> QAIHubConverter:
        """Initialize and return a Converter instance.

        Returns:
            QAIHubConverter: Initialized Converter instance.
        """
        return QAIHubConverter()

    def benchmarker(self) -> QAIHubBenchmarker:
        """Initialize and return a Benchmarker instance.

        Returns:
            QAIHubBenchmarker: Initialized Benchmarker instance.
        """
        return QAIHubBenchmarker()



import configparser
import os
from enum import Enum
from pathlib import Path

from dotenv import find_dotenv, load_dotenv
from loguru import logger

BASE_DIR = Path(__file__).resolve().parent
config_parser = configparser.ConfigParser()
DEPLOYMENT_MODE = os.getenv("DEPLOYMENT_MODE", "v2-prod-cloud")
logger.info(f"Read {DEPLOYMENT_MODE} config")
config_parser.read(f"{BASE_DIR}/configs/config-{DEPLOYMENT_MODE.lower()}.ini")


class EnvironmentType(str, Enum):
    V2_PROD_CLOUD = "v2-prod-cloud"
    V2_PROD_ON_PREM = "v2-prod-on-prem"
    V2_STAGING_CLOUD = "v2-staging-cloud"
    V2_STAGING_ON_PREM = "v2-staging-on-prem"
    V2_DEV_CLOUD = "v2-dev-cloud"
    V2_DEV_ON_PREM = "v2-dev-on-prem"


class Module(str, Enum):
    AUTH = "AUTH"
    COMPRESSOR = "COMPRESSOR"
    LAUNCHER = "LAUNCHER"
    NP = "NP"
    TAO = "TAO"


class EndPointProperty(str, Enum):
    HOST = "HOST"
    PORT = "PORT"
    URI_PREFIX = "URI_PREFIX"


class Config:
    def __init__(self, module: Module):
        self.ENVIRONMENT_TYPE = EnvironmentType(DEPLOYMENT_MODE.lower())
        self.MODULE = module

        dotenv_path = find_dotenv(filename="netspresso.env")
        if dotenv_path:
            load_dotenv(dotenv_path)

        if self.MODULE == Module.TAO:
            self.HOST = config_parser[self.MODULE][EndPointProperty.HOST]
            self.PORT = int(config_parser[self.MODULE][EndPointProperty.PORT])
            self.URI_PREFIX = config_parser[self.MODULE][EndPointProperty.URI_PREFIX]
        else:
            self.HOST = os.environ.get("HOST", config_parser[Module.NP][EndPointProperty.HOST])
            self.PORT = int(os.environ.get("PORT", config_parser[Module.NP][EndPointProperty.PORT]))
            self.URI_PREFIX = config_parser[f"NP.{self.MODULE}"][EndPointProperty.URI_PREFIX]

    def is_cloud(self) -> bool:
        return self.ENVIRONMENT_TYPE in [EnvironmentType.V2_PROD_CLOUD, EnvironmentType.V2_STAGING_CLOUD, EnvironmentType.V2_DEV_CLOUD]



from dataclasses import dataclass
from enum import Enum
from typing import List, Optional


class ComputeUnit(str, Enum):
    ALL = "all"
    NPU = "npu"
    GPU = "gpu"
    CPU = "cpu"


@dataclass
class CommonOptions:
    compute_unit: Optional[List[ComputeUnit]] = None



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from loguru import logger
from qai_hub.client import Dataset, Device, InferenceJob, ProfileJob
from qai_hub.public_rest_api import DatasetEntries

from netspresso.enums import Status
from netspresso.metadata.benchmarker import BenchmarkerMetadata
from netspresso.qai_hub.base import QAIHubBase
from netspresso.qai_hub.options import InferenceOptions, ProfileOptions
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class QAIHubBenchmarker(QAIHubBase):
    def download_benchmark_results(self, job: ProfileJob, artifacts_dir: str):
        results = job.download_results(artifacts_dir=artifacts_dir)

        return results

    def download_profile(self, job: ProfileJob):
        profile = job.download_profile()

        return profile

    def benchmark_model(
        self,
        input_model_path: Union[str, Path],
        target_device_name: Union[Device, List[Device]],
        options: Union[ProfileOptions, str] = ProfileOptions(),
        job_name: Optional[str] = None,
        retry: bool = True,
        wait_until_done: bool = True,
    ) -> Union[ProfileJob, List[ProfileJob]]:
        FileHandler.check_input_model_path(input_model_path)

        folder_path = Path(input_model_path).parent
        file_name = "benchmark"

        metadatas = []
        file_path = folder_path / f"{file_name}.json"
        if FileHandler.check_exists(file_path):
            metadatas = MetadataHandler.load_json(file_path)

        metadata = BenchmarkerMetadata()
        metadata.input_model_path = Path(input_model_path).resolve().as_posix()
        metadata.benchmark_task_info.device_name = target_device_name.name
        metadata.benchmark_task_info.display_device_name = target_device_name.name
        metadatas.append(metadata.asdict())
        MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name=file_name)

        try:
            model_type = hub.client._determine_model_type(model=input_model_path)
            framework = self.get_framework_by_model_type(model_type=model_type)
            display_framework = self.get_display_framework(framework)
            metadata.benchmark_task_info.framework = framework
            metadata.benchmark_task_info.display_framework = display_framework

            cli_string = options.to_cli_string() if isinstance(options, ProfileOptions) else options

            job: ProfileJob = hub.submit_profile_job(
                model=input_model_path,
                device=target_device_name,
                name=job_name,
                options=cli_string,
                retry=retry,
            )

            metadata.benchmark_task_info.benchmark_task_uuid = job.job_id
            metadatas[-1] = metadata.asdict()
            MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name=file_name)

            if wait_until_done:
                job = hub.get_job(job.job_id)
                status = job.wait()

                if status.success:
                    logger.info(f"{status.symbol} {status.state.name}")
                    profile = self.download_profile(job=job)
                    metadata.benchmark_result.latency = profile["execution_summary"]["estimated_inference_time"] / 1000
                    metadata.benchmark_result.memory_footprint = profile["execution_summary"][
                        "estimated_inference_peak_memory"
                    ]
                    metadata.status = Status.COMPLETED
                else:
                    logger.info(f"{status.symbol} {status.state}: {status.message}")
                    metadata.status = Status.ERROR

            metadatas[-1] = metadata.asdict()
            MetadataHandler.save_json(data=metadatas, folder_path=folder_path, file_name=file_name)

            return metadata

        except KeyboardInterrupt:
            metadata.status = Status.STOPPED
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=folder_path, file_name=file_name)

    def inference_model(
        self,
        input_model_path: Union[str, Path],
        target_device_name: Union[Device, List[Device]],
        inputs: Union[Dataset, DatasetEntries, str],
        job_name: Optional[str] = None,
        options: Union[InferenceOptions, str] = InferenceOptions(),
        retry: bool = True,
        wait_until_done: bool = True,
    ) -> Union[InferenceJob, List[InferenceJob]]:

        cli_string = options.to_cli_string() if isinstance(options, InferenceOptions) else options

        job: InferenceJob = hub.submit_inference_job(
            model=input_model_path,
            device=target_device_name,
            inputs=inputs,
            name=job_name,
            options=cli_string,
            retry=retry,
        )

        if wait_until_done:
            job = hub.get_job(job.job_id)
            status = job.wait()

            if status.success:
                logger.info(f"{status.symbol} {status.state.name}")
            else:
                logger.info(f"{status.symbol} {status.state}: {status.message}")

        return job



from pathlib import Path
from typing import List, Optional, Union

import qai_hub as hub
from loguru import logger
from qai_hub.client import CompileJob, Dataset, Device, InputSpecs
from qai_hub.public_rest_api import DatasetEntries

from netspresso.enums import Status
from netspresso.metadata.converter import ConverterMetadata
from netspresso.qai_hub.base import QAIHubBase
from netspresso.qai_hub.options import CompileOptions
from netspresso.utils import FileHandler
from netspresso.utils.metadata import MetadataHandler


class QAIHubConverter(QAIHubBase):
    def transform_shape(self, data):
        for value in data.values():
            shape, _ = value
            return {"batch": shape[0], "channel": shape[1], "dimension": list(shape[2:])}

    def convert_image_dict_to_list(self, image_dict):
        result = []
        for key, value in image_dict.items():
            batch, channel, *dimension = value
            result.append({
                "name": key,
                "batch": batch,
                "channel": channel,
                "dimension": dimension
            })
        return result

    def dict_to_tuple(self, data):
        batch = data.get("batch")
        channel = data.get("channel")
        dimension = data.get("dimension", [])

        if batch is not None and channel is not None and isinstance(dimension, list):
            return (batch, channel, *dimension)
        else:
            raise ValueError("Invalid input data format")

    def convert_model(
        self,
        input_model_path: Union[str, Path],
        output_dir: str,
        target_device_name: Union[Device, List[Device]],
        input_shapes: Optional[InputSpecs] = None,
        options: Union[CompileOptions, str] = CompileOptions(),
        job_name: Optional[str] = None,
        single_compile: bool = True,
        calibration_data: Union[Dataset, DatasetEntries, str, None] = None,
        retry: bool = True,
        wait_until_done: bool = True,
    ) -> Union[ConverterMetadata, List[ConverterMetadata]]:

        output_dir = FileHandler.create_unique_folder(folder_path=output_dir)
        default_model_path = (Path(output_dir) / f"{Path(output_dir).name}.ext").resolve()
        metadata = ConverterMetadata()
        metadata.input_model_path = Path(input_model_path).resolve().as_posix()
        extension = self.get_source_extension(model_path=input_model_path)
        metadata.model_info.framework = self.get_framework(extension=extension)

        MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

        try:
            target_extension = self.get_target_extension(runtime=options.target_runtime)
            converted_model_path = default_model_path.with_suffix(target_extension).as_posix()

            cli_string = options.to_cli_string() if isinstance(options, CompileOptions) else options

            job = hub.submit_compile_job(
                model=input_model_path,
                device=target_device_name,
                name=job_name,
                input_specs=input_shapes,
                options=cli_string,
                single_compile=single_compile,
                calibration_data=calibration_data,
                retry=retry,
            )

            framework = self.get_framework_by_runtime(options.target_runtime)
            display_framework = self.get_display_framework(framework)

            # metadata.model_info.input_shapes = [input_shape]
            metadata.model_info.input_shapes = self.convert_image_dict_to_list(input_shapes)
            metadata.model_info.data_type = job.shapes["image"][1]
            metadata.convert_task_info.convert_task_uuid = job.job_id
            metadata.convert_task_info.input_model_uuid = job.model.model_id
            metadata.convert_task_info.device_name = target_device_name.name
            metadata.convert_task_info.display_device_name = target_device_name.name
            metadata.convert_task_info.framework = framework
            metadata.convert_task_info.display_framework = display_framework

            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            if wait_until_done:
                job: CompileJob = hub.get_job(job.job_id)
                status = job.wait()

                if status.success:
                    logger.info(f"{status.symbol} {status.state.name}")
                    self.download_model(job=job, filename=converted_model_path)
                    target_model = job.get_target_model()
                    metadata.convert_task_info.output_model_uuid = target_model.model_id
                    metadata.converted_model_path = converted_model_path
                    metadata.convert_task_info.data_type = job.target_shapes["image"][1]
                    metadata.available_options = job.compatible_devices
                    metadata.status = Status.COMPLETED
                else:
                    logger.info(f"{status.symbol} {status.state}: {status.message}")
                    metadata.status = Status.ERROR
                    metadata.update_message(exception_detail=status.message)

            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

            return metadata

        except KeyboardInterrupt:
            metadata.status = Status.STOPPED
            MetadataHandler.save_json(data=metadata.asdict(), folder_path=output_dir)

    def download_model(self, job: CompileJob, filename: str):
        job.download_target_model(filename=filename)
